﻿using UnityEngine;
using System.Collections;

public class GameControllerScript : MonoBehaviour {

	private string[] levels = new string[] {
		"Level01",
		"Level02"
	};

	private GameObject textLost;
	private GameObject textWon;
	private GameObject player;

	private bool waitForNextLevel = false;
	private bool waitForReplay = false;
	private int levelPointer = 0;

	// Use this for initialization
	void Start () {
		textLost = GameObject.Find("UI Root/TextLost").gameObject;
		textWon = GameObject.Find("UI Root/TextWon").gameObject;
		player = GameObject.Find("Player").gameObject;
		textLost.SetActive(false);
		textWon.SetActive(false);
	}

	public void LoadLevel(int level){
		if(textLost != null && textWon != null){
			textLost.SetActive(false);
			textWon.SetActive(false);
		}
		levelPointer = level;
		Application.LoadLevel(levels[level]);
	}

	public void PlayerKilled(){
		if(!waitForNextLevel){
			Destroy(player);
			textLost.SetActive(true);
			waitForReplay = true;
		}
	}

	public void PlayerReachedFinish(){
		if(!waitForReplay){
			textWon.SetActive(true);
			waitForNextLevel = true;
		}
	}

	public void LoadNextLevel(){
		levelPointer++;
		if(levelPointer >= levels.GetLength(0))
			levelPointer = 0;
		LoadLevel(levelPointer);
	}

	void Update () {
		if(Input.anyKeyDown){
			if(waitForReplay){
				LoadLevel(levelPointer);
			}else if(waitForNextLevel){
				LoadNextLevel();
			}
		}
	}
	
	void OnMouseDown(){
		if(waitForReplay){
			LoadLevel(levelPointer);
		}else if(waitForNextLevel){
			LoadNextLevel();
		}
	}
	
	
}
